library flutter_swipping_button;

import 'package:flutter/material.dart';

class SwipableButton extends StatefulWidget {
  @override
  _SwipableButtonState createState() => _SwipableButtonState();
  final Widget child;

  /// The Height of the widget that will be drawn, required.
  final double height;

  /// The `VoidCallback` that will be called once a swipe with certain percentage is detected.
  final VoidCallback onSwipeCallback;

  /// The decimal percentage of swiping in order for the callbacks to get called, defaults to 0.75 (75%) of the total width of the children.
  final double swipePercentageNeeded;

  final bool returnToInitialPosition;

  final Duration delay;

  SwipableButton(
      {Key key,
      @required this.child,
      @required this.height,
      @required this.onSwipeCallback,
      this.returnToInitialPosition = false,
      this.delay,
      this.swipePercentageNeeded = 0.75})
      : assert(child != null &&
            onSwipeCallback != null &&
            swipePercentageNeeded <= 1.0 &&
            (returnToInitialPosition && delay != null)),
        super(key: key);
}

class _SwipableButtonState extends State<SwipableButton>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;

  double _dxStartPosition = 0.0;
  double _dxEndsPosition = 0.0;

  @override
  void initState() {
    super.initState();
    //TODO: make duration variable
    _controller = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 300))
      ..addListener(() {
        setState(() {});
      });

    _controller.value = 1.0;
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onPanStart: (details) {
        setState(() {
          _dxStartPosition = details.localPosition.dx;
        });
      },
      onPanUpdate: (details) {
        final widgetSize = context.size.width;
        //TODO: make swipable length variable
        final minimumXToStartSwiping = widgetSize * 0.25;
        if (_dxStartPosition <= minimumXToStartSwiping) {
          setState(() {
            _dxEndsPosition = details.localPosition.dx;
          });

          final widgetSize = context.size.width;
          _controller.value = 1 - ((details.localPosition.dx) / widgetSize);
        }
      },
      onPanEnd: (details) async {
        final delta = _dxEndsPosition - _dxStartPosition;
        final widgetSize = context.size.width;
        final deltaNeededToBeSwiped = widgetSize * widget.swipePercentageNeeded;
        //TODO: make animation more customizable
        if (delta > deltaNeededToBeSwiped) {
          _controller.animateTo(0.0,
              duration: Duration(milliseconds: 300),
              curve: Curves.fastOutSlowIn);
          widget.onSwipeCallback();
          if (widget.returnToInitialPosition) {
            Future.delayed(
                widget.delay,
                () => {
                      _controller.animateTo(1.0,
                          duration: Duration(milliseconds: 300),
                          curve: Curves.fastOutSlowIn)
                    });
          }
        } else {
          _controller.animateTo(1.0,
              duration: Duration(milliseconds: 300),
              curve: Curves.fastOutSlowIn);
        }
      },
      child: Container(
        height: widget.height,
        child: Align(
          alignment: Alignment.centerRight,
          child: FractionallySizedBox(
            widthFactor: _controller.value,
            heightFactor: 1.0,
            child: widget.child,
          ),
        ),
      ),
    );
  }
}
